##Library Header
# $Id: $
# Copyright  2003-2009 by IXIA
# All Rights Reserved.
#
# Name:
#    ixia_pim_api.tcl
#
# Purpose:
#     A script development library containing PIM APIs for test automation
#     with the Ixia chassis.
#
# Author:
#    T. Kong
#
# Usage:
#    package require Ixia
#
# Description:
#    The procedures contained within this library include:
#
#    - emulation_pim_config
#    - emulation_pim_group_config
#    - emulation_pim_control
#    - emulation_pim_info
#
# Requirements:
#     ixiaapiutils.tcl , a library containing TCL utilities
#     parseddashedargs.tcl , a library containing the proceDescr and
#     parsedashedargds.tcl
#
# Variables:
#
# Keywords:
#
# Category:
#
################################################################################
#                                                                              #
#                                LEGAL  NOTICE:                                #
#                                ==============                                #
# The following code and documentation (hereinafter "the script") is an        #
# example script for demonstration purposes only.                              #
# The script is not a standard commercial product offered by Ixia and have     #
# been developed and is being provided for use only as indicated herein. The   #
# script [and all modifications, enhancements and updates thereto (whether     #
# made by Ixia and/or by the user and/or by a third party)] shall at all times #
# remain the property of Ixia.                                                 #
#                                                                              #
# Ixia does not warrant (i) that the functions contained in the script will    #
# meet the user's requirements or (ii) that the script will be without         #
# omissions or error-free.                                                     #
# THE SCRIPT IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, AND IXIA        #
# DISCLAIMS ALL WARRANTIES, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE,          #
# INCLUDING BUT NOT LIMITED TO ANY WARRANTY OF MERCHANTABILITY AND FITNESS FOR #
# A PARTICULAR PURPOSE OR OF NON-INFRINGEMENT.                                 #
# THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SCRIPT  IS WITH THE #
# USER.                                                                        #
# IN NO EVENT SHALL IXIA BE LIABLE FOR ANY DAMAGES RESULTING FROM OR ARISING   #
# OUT OF THE USE OF, OR THE INABILITY TO USE THE SCRIPT OR ANY PART THEREOF,   #
# INCLUDING BUT NOT LIMITED TO ANY LOST PROFITS, LOST BUSINESS, LOST OR        #
# DAMAGED DATA OR SOFTWARE OR ANY INDIRECT, INCIDENTAL, PUNITIVE OR            #
# CONSEQUENTIAL DAMAGES, EVEN IF IXIA HAS BEEN ADVISED OF THE POSSIBILITY OF   #
# SUCH DAMAGES IN ADVANCE.                                                     #
# Ixia will not be required to provide any software maintenance or support     #
# services of any kind (e.g., any error corrections) in connection with the    #
# script or any part thereof. The user acknowledges that although Ixia may     #
# from time to time and in its sole discretion provide maintenance or support  #
# services for the script, any such services are subject to the warranty and   #
# damages limitations set forth herein and will not obligate Ixia to provide   #
# any additional maintenance or support services.                              #
#                                                                              #
################################################################################

proc ::ixia::emulation_pim_config { args } {
    variable executeOnTclServer
    
    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_pim_config $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    variable bgp_neighbor_handles_array
    variable bgp_route_handles_array
    variable new_ixnetwork_api

    ::ixia::utrackerLog $procName $args

    # Arguments
    set man_args {
        -port_handle    REGEXP  ^[0-9]+/[0-9]+/[0-9]+$
        -mode           CHOICES create modify delete disable enable enable_all disable_all
                        DEFAULT create
    }

    set opt_args {
        -handle
        -interface_handle
        -pim_mode                   CHOICES sm ssm
                                    DEFAULT sm
        -type                       CHOICES remote_rp
        -ip_version                 CHOICES 4 6
                                    DEFAULT 4
        -bootstrap_enable           CHOICES 0 1
                                    DEFAULT 0
        -bootstrap_support_unicast  CHOICES 0 1
                                    DEFAULT 1
        -bootstrap_hash_mask_len    RANGE   0-128
        -bootstrap_interval         RANGE   0-65535
                                    DEFAULT 60
        -bootstrap_priority         RANGE   0-255
                                    DEFAULT 64
        -bootstrap_timeout          RANGE   0-65535
                                    DEFAULT 130
        -count                      NUMERIC
                                    DEFAULT 1
        -intf_ip_addr               IP
        -intf_ip_addr_step          IP
        -intf_ip_prefix_length      RANGE 1-128
        -intf_ip_prefix_len         RANGE 1-128
        -learn_selected_rp_set      CHOICES 0 1
                                    DEFAULT 1
        -discard_learnt_rp_info     CHOICES 0 1
                                    DEFAULT 0
        -router_id                  IP
        -router_id_step             IP
        -gateway_intf_ip_addr       IP
        -gateway_intf_ip_addr_step  IP
        -neighbor_intf_ip_addr      IP
        -neighbor_intf_ip_addr_step IP
        -dr_priority                NUMERIC
                                    DEFAULT 0
        -bidir_capable              CHOICES 0 1
                                    DEFAULT 0
        -hello_interval             NUMERIC
                                    DEFAULT 30
        -hello_holdtime             NUMERIC
                                    DEFAULT 105
        -join_prune_interval        NUMERIC
                                    DEFAULT 60
        -join_prune_holdtime        NUMERIC
                                    DEFAULT 180
        -prune_delay_enable         CHOICES 0 1
                                    DEFAULT 0
        -prune_delay                RANGE 100-32767
                                    DEFAULT 500
        -override_interval          RANGE 100-65535
                                    DEFAULT 2500
        -vlan_id                    RANGE 0-4096
        -vlan_id_mode               CHOICES fixed increment
                                    DEFAULT increment
        -vlan_id_step               RANGE 0-4096
                                    DEFAULT 1
        -vlan_user_priority         RANGE 0-7
                                    DEFAULT 0
        -vlan_cfi                   CHOICES 0 1
        -mvpn_enable                CHOICES 0 1
                                    DEFAULT 0
        -mvpn_pe_count              NUMERIC
                                    DEFAULT 1
        -mvpn_pe_ip                 IP
        -mvpn_pe_ip_incr            IP
        -mvrf_count                 NUMERIC
        -mvrf_unique                CHOICES 0 1
                                    DEFAULT 0
        -default_mdt_ip             IP
        -default_mdt_ip_incr        IP
        -gre_checksum_enable        CHOICES 0 1
                                    DEFAULT 0
        -gre_key_enable             CHOICES 0 1
                                    DEFAULT 0
        -gre_key_in                 RANGE 0-4294967295
                                    DEFAULT 0
        -gre_key_out                RANGE 0-4294967295
                                    DEFAULT 0
        -reset
        -generation_id_mode         CHOICES increment random constant
        -prune_delay_tbit           CHOICES 0 1
        -send_generation_id         CHOICES 0 1
        -mac_address_init           MAC
        -mac_address_step           MAC
                                    DEFAULT 0000.0000.0001
        -vlan                       CHOICES 0 1
        -writeFlag                  CHOICES write nowrite
                                    DEFAULT write
        -no_write                   FLAG
        -gre                        CHOICES 0 1
                                    DEFAULT 0
        -gre_enable                 CHOICES 0 1
                                    DEFAULT 0
        -gre_unique                 CHOICES 0 1
                                    DEFAULT 1
        -gre_dst_ip_addr            IP
        -gre_count                  NUMERIC
                                    DEFAULT 1
        -gre_ip_addr                IP
        -gre_ip_addr_step           IP
        -gre_ip_addr_lstep          IP
        -gre_ip_addr_cstep          IP
        -gre_ip_prefix_length       RANGE   1-128
        -gre_dst_ip_addr_step       IP
        -gre_dst_ip_addr_lstep      IP
        -gre_dst_ip_addr_cstep      IP
        -gre_key_in_step            RANGE 0-4294967295
                                    DEFAULT 0
        -gre_key_out_step           RANGE 0-4294967295
                                    DEFAULT 0
        -gre_src_ip_addr_mode       CHOICES routed connected
                                    DEFAULT connected
        -gre_seq_enable             CHOICES 0 1
                                    DEFAULT 0
        -loopback_count             NUMERIC
                                    DEFAULT 0
        -loopback_ip_address        IP
        -loopback_ip_address_step   IP
        -loopback_ip_address_cstep  IP
    }
    
    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_pim_config $args $man_args $opt_args]
        if {[keylget returnList status] == $::FAILURE} {
            keylset returnList log "ERROR in $procName:\
                    [keylget returnList log]"
        }
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}

proc ::ixia::emulation_pim_group_config { args } {
    variable executeOnTclServer
    variable new_ixnetwork_api

    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_pim_group_config $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    variable pimsm_handles_array
    variable multicast_group_array
    variable multicast_source_array

    ::ixia::utrackerLog $procName $args

    keylset returnList status $::SUCCESS

    # Arguments
    set opt_args {
        -mode                           CHOICES create delete modify clear_all
                                        DEFAULT create
        -session_handle
        -group_pool_handle
        -source_pool_handle
        -handle
        -adv_hold_time                  RANGE 2-65535
                                        DEFAULT 150
        -back_off_interval              RANGE 0-255
                                        DEFAULT 3
        -crp_ip_addr                    IP
        -rp_ip_addr                     IP
        -rp_ip_addr_step                IP
        -group_pool_mode                CHOICES send register candidate_rp
        -join_prune_aggregation_factor  NUMERIC
        -wildcard_group                 CHOICES 0 1
        -s_g_rpt_group                  CHOICES 0 1
        -rate_control                   CHOICES 0 1
        -interval                       RANGE 50-1000
        -join_prune_per_interval        NUMERIC
        -register_per_interval          NUMERIC
        -register_stop_per_interval     NUMERIC
        -flap_interval                  RANGE 1-65535
        -periodic_adv_interval          RANGE 1-65535
                                        DEFAULT 60
        -pri_change_interval            RANGE 1-65535
                                        DEFAULT 60
        -pri_type                       CHOICES same incremental random
        -pri_value                      RANGE 0-255
                                        DEFAULT 192
        -register_tx_iteration_gap      RANGE 100-2147483647
        -register_stop_trigger_count    RANGE 1-127
        -register_udp_destination_port  RANGE 1-65535
        -register_udp_source_port       RANGE 1-65535
        -register_triggered_sg          CHOICES 0 1
        -router_count                   RANGE 1-65535
                                        DEFAULT 1
        -spt_switchover                 CHOICES 0 1
        -source_group_mapping           CHOICES fully_meshed one_to_one
        -switch_over_interval           RANGE 1-65535
        -send_null_register             CHOICES 0 1
                                        DEFAULT 0
        -trigger_crp_msg_count          RANGE 1-3
                                        DEFAULT 3
        -writeFlag                      CHOICES write nowrite
                                        DEFAULT write
        -no_write                       FLAG
        -default_mdt_mode               CHOICES neighbor auto
                                        DEFAULT neighbor
    }
    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_pim_group_config $args $opt_args]
        if {[keylget returnList status] == $::FAILURE} {
            keylset returnList log "ERROR in $procName:\
                    [keylget returnList log]"
        }
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}

proc ::ixia::emulation_pim_control { args } {
    variable executeOnTclServer
    variable new_ixnetwork_api

    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_pim_control $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    variable pimsm_handles_array

    ::ixia::utrackerLog $procName $args

    keylset returnList status $::SUCCESS

    # Arguments
    set man_args {
            -mode           CHOICES stop start restart
    }

    set opt_args {
            -port_handle    REGEXP  ^[0-9]+/[0-9]+/[0-9]+$
            -handle
            -flap           CHOICES 0 1
            -flap_interval  RANGE 1-65535
    }

	if {[isUNIX] && [info exists ::ixTclSvrHandle]} {
        set retValueClicks [eval "::ixia::SendToIxTclServer $::ixTclSvrHandle {clock clicks}"]
        set retValueSeconds [eval "::ixia::SendToIxTclServer $::ixTclSvrHandle {clock seconds}"]
    } else {
        set retValueClicks [clock clicks]
        set retValueSeconds [clock seconds]
    }

    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_pim_control $args $man_args $opt_args]
        if {[keylget returnList status] == $::FAILURE} {
            keylset returnList log "ERROR in $procName:\
                    [keylget returnList log]"
        }
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}

    return $returnList
}

proc ::ixia::emulation_pim_info { args } {
    variable new_ixnetwork_api
    variable executeOnTclServer

    set procName [lindex [info level [info level]] 0]

    ::ixia::logHltapiCommand $procName $args

    set procName [lindex [info level [info level]] 0]

    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_pim_info $args\}]
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }

    ::ixia::utrackerLog $procName $args
    
    set man_args {
        -handle
    }
    set opt_args {
        -mode       CHOICES aggregate learned_crp
                    DEFAULT aggregate
    }
        
    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        
        set returnList [::ixia::ixnetwork_pim_info $args $man_args $opt_args]
        if {[keylget returnList status] == $::FAILURE} {
            keylset returnList status $::FAILURE
            keylset returnList log "ERROR in $procName: \
                    [keylget returnList log]"
        }
    } else {
        set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
    }
    return $returnList
}
